@@ -11,7 +11,7 @@ from astropy.wcs import WCS
 from .model import EmissionFitting
 from .tools import label_decomposition, LineFinder, UNITS_LATEX_DICT, latex_science_float, DISPERSION_UNITS,\
                    FLUX_DENSITY_UNITS, unit_convertor
-from .plots import LiMePlots, STANDARD_PLOT, STANDARD_AXES, colorDict, save_close_fig_swicth
+from .plots import LiMePlots, IntMaskInspector, STANDARD_PLOT, STANDARD_AXES, colorDict, save_close_fig_swicth
 from .io import _LOG_DTYPES_REC, _LOG_EXPORT, _LOG_COLUMNS, load_lines_log, save_line_log
 from .model import gaussian_profiles_computation, linear_continuum_computation
 
@@ -32,7 +32,7 @@ if mplcursors_check:
     popupProps['bbox']['alpha'] = 0.9
 
 
-class Spectrum(EmissionFitting, LiMePlots, LineFinder):
+class Spectrum(EmissionFitting, LiMePlots, IntMaskInspector, LineFinder):
 
     """
     This class defines a spectrum object from which the lines can be measured. The required inputs for the spectrum definition
@@ -45,7 +45,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
     the class functions.
 
     The user can also provide a two value array with the same wavelength limits. This array must be in the
-    same units and frame of reference as the ``.wave``.
+    same units and _frame of reference as the ``.wave``.
 
     The user can also include the spectrum instrumental FWHM so it can be taken into account during the measurements.
 
@@ -85,6 +85,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         LineFinder.__init__(self)
         EmissionFitting.__init__(self)
         LiMePlots.__init__(self)
+        IntMaskInspector.__init__(self)
 
         # Class attributes
         self.wave = None
@@ -136,6 +137,15 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
             if input_err is not None:
                 self.err_flux = self.err_flux / self.norm_flux
 
+        # Masked the arrays if requested
+        if pixel_mask is not None:
+            self.wave = np.ma.masked_array(self.wave, pixel_mask)
+            self.wave_rest = np.ma.masked_array(self.wave_rest, pixel_mask)
+            self.flux = np.ma.masked_array(self.flux, pixel_mask)
+
+            if self.err_flux is not None:
+                self.err_flux = np.ma.masked_array(self.err_flux, pixel_mask)
+
         # Check for dimensions and nan entries
         for arg in ['wave', 'flux', 'err_flux']:
             arg_array = self.__getattribute__(arg)
@@ -152,14 +162,6 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
                 if arg in ['wave', 'flux']:
                     _logger.warning(f'No {arg} array introduced. This may bring issues in the measurement.')
 
-        # Masked the arrays if requested
-        if pixel_mask is not None:
-            self.wave = np.ma.masked_array(self.wave, pixel_mask)
-            self.wave_rest = np.ma.masked_array(self.wave_rest, pixel_mask)
-            self.flux = np.ma.masked_array(self.flux, pixel_mask)
-
-            if self.err_flux is not None:
-                self.err_flux = np.ma.masked_array(self.err_flux, pixel_mask)
 
         # Check for masked arrays
         check_mask = np.zeros(3).astype(bool)
@@ -170,7 +172,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         if np.any(check_mask):
             self._masked_inputs = True
             if ~np.all(check_mask):
-                _logger.warning(f'Make sure *both* your input wavelength and flux array are masked arrays')
+                _logger.warning(f'Make sure *all* your input wavelength, flux and uncertainty are masked arrays')
 
         # Generate empty dataframe to store measurement use cwd as default storing folder
         self.log = pd.DataFrame(np.empty(0, dtype=_LOG_DTYPES_REC))
@@ -186,7 +188,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         ion and wavelength (with units) separated by an underscore, i.e. O3_5007A.
 
         The location mask consists in a 6 values array with the wavelength boundaries for the line location and two
-        adjacent continua. These wavelengths must be sorted by increasing order and in the rest frame.
+        adjacent continua. These wavelengths must be sorted by increasing order and in the rest _frame.
 
         The user can specify the properties of the fitting: Number of components and parameter boundaries. Please check
         the documentation for the complete description.
@@ -230,8 +232,8 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
 
         # Security checks for the mask wavelengths
         assert np.all(np.diff(mask) >= 0), f'\n- Error: the {line} mask is not sorted'
-        assert self.wave_rest[0] < mask[0], f'\n- Error: the {line} mask low mask limit (w1 = {mask[0]:.2f}) is below the spectrum rest frame limit (w_min = {self.wave_rest[0]:.2f})'
-        assert self.wave_rest[-1] > mask[-1], f'\n- Error: the {line} mask up mask limit (w6 = {mask[-1]:.2f}) is above the spectrum rest frame limit (w_min = {self.wave_rest[-1]:.2f})'
+        # assert self.wave_rest[0] < mask[0], f'\n- Error: the {line} mask low mask limit (w1 = {mask[0]:.2f}) is below the spectrum rest _frame limit (w_min = {self.wave_rest[0]:.2f})'
+        # assert self.wave_rest[-1] > mask[-1], f'\n- Error: the {line} mask up mask limit (w6 = {mask[-1]:.2f}) is above the spectrum rest _frame limit (w_min = {self.wave_rest[-1]:.2f})'
 
         # For security previous measurement is cleared and a copy of the user configuration is used
         self.clear_fit()
@@ -293,7 +295,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         x_array = self.wave[idcsLine]
         y_array = self.flux[idcsLine]
         w_array = 1.0 / self.err_flux[idcsLine] if self.err_flux is not None else np.full(x_array.size, 1.0 / self.std_cont)
-        self.gauss_lmfit(self.line, x_array, y_array, w_array, fit_conf, self.log, z_obj=self.redshift)
+        self.gauss_lmfit(self.line, x_array, y_array, w_array, fit_conf, z_obj=self.redshift)
 
         # Safe the results to log DF
         self.results_to_database(self.line, self.log, fit_conf)
@@ -325,7 +327,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         :param log_scale: Check for the scale of the flux (vertical) axis in the plot. The default value is ``True``
         :type log_scale: bool, optional
 
-        :param frame: Frame of reference for the plot. The default value is the observation frame ``frame='obs'``
+        :param frame: Frame of reference for the plot. The default value is the observation _frame ``_frame='obs'``
         :type frame: str, optional
 
         :param output_address: Output address for the measurement report and/or plot. If provided the results will be stored
@@ -427,14 +429,14 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
 
                         # Warning overwritten existing configuration
                         if param_label_child in user_conf:
-                            _logger.warning('{param_label_child} overwritten by {parent_label} kinematics in configuration input')
+                            _logger.warning(f'{param_label_child} overwritten by {parent_label} kinematics in configuration input')
 
                         # Case where parent and child are in blended group
                         if parent_label in childs_list:
-                            param_label_parent = f'{parent_label.replace(".", "dot")}_{param_ext}'
+                            param_label_parent = f'{parent_label}_{param_ext}'
                             param_expr_parent = f'{wtheo_child / wtheo_parent:0.8f}*{param_label_parent}'
 
-                            user_conf[param_label_child.replace(".", "dot")] = {'expr': param_expr_parent}
+                            user_conf[param_label_child] = {'expr': param_expr_parent}
 
                         # Case we want to copy from previously measured line
                         else:
@@ -446,8 +448,8 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
                             else:
                                 param_value = wtheo_child / wtheo_parent * sigma_parent
 
-                            user_conf[param_label_child.replace(".", "dot")] = {'value': param_value[0], 'vary': False}
-                            user_conf[f'{param_label_child.replace(".", "dot")}_err'] = param_value[1]
+                            user_conf[param_label_child] = {'value': param_value[0], 'vary': False}
+                            user_conf[f'{param_label_child}_err'] = param_value[1]
 
         return
 
@@ -490,7 +492,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         if units_wave is not None:
 
             if units_wave in DISPERSION_UNITS:
-                self.wave = unit_convertor(self.units_wave, units_wave, wave_array=self.wave)
+                self.wave = unit_convertor(self.units_wave, units_wave, wave_array=self.wave, mask_check=np.ma.is_masked(self.wave))
                 self.wave_rest = self.wave/(1+self.redshift)
             else:
                 _logger.warning(f'- Dispersion units {units_wave} not recognized for conversion. '
@@ -499,12 +501,17 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
             # Reflect the new units
             self.units_wave = units_wave
 
-
         if units_flux is not None:
 
             if units_flux in FLUX_DENSITY_UNITS:
                 self.flux = unit_convertor(self.units_flux, units_flux, wave_array=self.wave, flux_array=self.flux,
-                                           dispersion_units=self.units_wave)
+                                           dispersion_units=self.units_wave, mask_check=np.ma.is_masked(self.flux))
+
+                if self.err_flux is not None:
+                    self.err_flux = unit_convertor(self.units_flux, units_flux, wave_array=self.wave,
+                                                   flux_array=self.err_flux, dispersion_units=self.units_wave,
+                                                   mask_check=np.ma.is_masked(self.err_flux))
+
             else:
                 _logger.warning(f'- Dispersion units {units_flux} not recognized for conversion. '
                                 f'Please use {FLUX_DENSITY_UNITS} to convert from {self.units_flux}')
@@ -512,14 +519,17 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
             # Reflect the new units
             self.units_flux = units_flux
 
+        # Switch the normalization
         if norm_flux is not None:
 
             # Undo previous normalization
             self.flux = self.flux * self.norm_flux
+            self.err_flux = self.err_flux * self.norm_flux if self.err_flux is not None else None
 
             # New normalization
             self.norm_flux = norm_flux
             self.flux = self.flux/self.norm_flux
+            self.err_flux = self.err_flux/self.norm_flux if self.err_flux is not None else None
 
         return
 
@@ -531,8 +541,8 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
 class MaskInspector(Spectrum):
 
     """
-    This class plots the masks from the ``log_address`` as a grid for the input spectrum. Clicking and
-    dragging the mouse within a line cell will update the line band region, both in the plot and the ``log_address``
+    This class plots the masks from the ``_log_address`` as a grid for the input spectrum. Clicking and
+    dragging the mouse within a line cell will update the line band region, both in the plot and the ``_log_address``
     file provided.
 
     Assuming that the band wavelengths `w1` and `w2` specify the adjacent blue (left continuum), the `w3` and `w4`
@@ -609,8 +619,7 @@ class MaskInspector(Spectrum):
 
         # Lines log not provide code ends
         else:
-            print(f'- ERROR: No lines log provided by the user nor can be found the lines log file at address:'
-                  f' {log_address}')
+            _logger.warning(f'No lines log file found at {log_address}. Leaving the script')
             exit()
 
         # Only plotting the lines in the lines interval
@@ -624,7 +633,6 @@ class MaskInspector(Spectrum):
             if isinstance(lines_interval[0], str):
                 n_lines = len(lines_interval)
                 self.target_lines = np.array(lines_interval, ndmin=1)
-
             # Array of integers
             else:
                 n_lines = lines_interval[1] - lines_interval[0]
@@ -659,8 +667,6 @@ class MaskInspector(Spectrum):
             try:
                 manager = plt.get_current_fig_manager()
                 manager.window.showMaximized()
-
-            # TODO add exception
             except:
                 print()
 
@@ -717,7 +723,7 @@ class MaskInspector(Spectrum):
             waveLine, fluxLine = self.wave_rest[idcsLineArea], self.flux[idcsLineArea]
 
             # Plot the line region
-            ax.step(waveLine, fluxLine)
+            ax.step(waveLine, fluxLine, where='mid')
 
             # Fill the user selections
             if non_nan == 2:
@@ -765,16 +771,18 @@ class MaskInspector(Spectrum):
         ax.update({'title': lineLabel})
         ax.yaxis.set_ticklabels([])
         ax.axes.yaxis.set_visible(False)
+        try:
+            idxPeakFlux = np.argmax(fluxPeak)
+            ax.set_ylim(ymin=np.min(fluxLine) / 5, ymax=fluxPeak[idxPeakFlux] * 1.25)
 
-        idxPeakFlux = np.argmax(fluxPeak)
-        ax.set_ylim(ymin=np.min(fluxLine) / 5, ymax=fluxPeak[idxPeakFlux] * 1.25)
-
-        if logscale == 'auto':
-            if fluxPeak[idxPeakFlux] > 5 * np.median(fluxLine):
-                ax.set_yscale('log')
-        else:
-            if logscale == 'log':
-                ax.set_yscale('log')
+            if logscale == 'auto':
+                if fluxPeak[idxPeakFlux] > 5 * np.median(fluxLine):
+                    ax.set_yscale('log')
+            else:
+                if logscale == 'log':
+                    ax.set_yscale('log')
+        except:
+            print(f'fail at {self.line}')
 
 
         return
@@ -981,7 +989,7 @@ class CubeInspector(Spectrum):
 
         """
 
-        #TODO add frame argument
+        #TODO add _frame argument
 
         # Assign attributes to the parent class
         super().__init__(input_wave=np.zeros(1), input_flux=np.zeros(1), redshift=redshift, norm_flux=1, units_wave=units_wave, units_flux=units_flux)
@@ -1159,7 +1167,7 @@ class CubeInspector(Spectrum):
                 # Plot all lines encountered in the voxel log
                 line_list = self.log.index.values
 
-                # Reference frame for the plot
+                # Reference _frame for the plot
                 wave_plot, flux_plot, z_corr, idcs_mask = self.frame_mask_switch(self.wave, flux_voxel, self.redshift, frame)
 
                 # Compute the individual profiles
